'use strict';

var _extends = require('babel-runtime/helpers/extends')['default'];

var _Object$assign = require('babel-runtime/core-js/object/assign')['default'];

var _getIterator = require('babel-runtime/core-js/get-iterator')['default'];

var _Object$keys = require('babel-runtime/core-js/object/keys')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.extractHandlers = extractHandlers;
exports.get = get;
exports.register = register;
exports.create = create;
exports.update = update;
exports.updateChildren = updateChildren;

var _reactLibInvariant = require('react/lib/invariant');

var _reactLibInvariant2 = _interopRequireDefault(_reactLibInvariant);

var assign = _Object$assign;

// Utilities

var handlerRE = /^on[A-Z]/g;

function extractHandlers(props) {
  var handlers = {};
  var rest = {};

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = _getIterator(_Object$keys(props)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var key = _step.value;

      if (key.match(handlerRE)) {
        handlers[key.slice(2, 3).toLowerCase() + key.slice(3)] = props[key];
      } else {
        rest[key] = props[key];
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator['return']) {
        _iterator['return']();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return { handlers: handlers, rest: rest };
}

// Definitions

var registry = {};

function get(type) {
  (0, _reactLibInvariant2['default'])(type in registry, 'No definition found for "' + type + '"');

  return registry[type];
}

function register(shortName, apiName) {
  var config = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var definition = _extends({ shortName: shortName, apiName: apiName }, config);

  registry[shortName] = definition;
  registry[apiName] = definition;

  return definition;
}

function create(type, props, handlers) {
  var definition = get(type);

  var view = definition.factory(props);

  for (var _name in handlers) {
    view.addEventListener(_name, handlers[_name]);
  }

  return view;
}

function update(view, props, handlers) {
  // TODO: manage handlers

  for (var key in props) {
    var nextValue = props[key];

    if (key === 'value') {
      var oldValue = view[key];

      if (nextValue === oldValue) {
        continue;
      }
    }

    view[key] = nextValue;
  }

  // view.applyProperties(props);
}

function updateChildren(view, children) {
  view.removeAllChildren();

  // NOTE: Sloooooooow...

  var i = 0;
  var l = children.length;

  for (; i < l; ++i) {
    if (children[i]) {
      view.add(children[i]);
    }
  }
}

// Built-ins

register('window', 'Titanium.UI.Window', {
  factory: function factory(props) {
    return Titanium.UI.createWindow(props);
  }
});

register('view', 'Titanium.UI.View', {
  factory: function factory(props) {
    return Titanium.UI.createView(props);
  }
});

register('button', 'Titanium.UI.Button', {
  factory: function factory(props) {
    return Titanium.UI.createButton(props);
  }
});

register('input', 'Titanium.UI.TextField', {
  factory: function factory(props) {
    return Titanium.UI.createTextField(props);
  }
});

register('textarea', 'Titanium.UI.TextArea', {
  factory: function factory(props) {
    return Titanium.UI.createTextArea(props);
  }
});

register('label', 'Titanium.UI.Label', {
  factory: function factory(props) {
    return Titanium.UI.createLabel(props);
  }
});

register('list', 'Titanium.UI.ListView', {
  factory: function factory(props) {
    return Titanium.UI.createListView(props);
  }
});

register('root', 'react.titanium.Root', {
  factory: function factory(props) {
    return {
      apiName: 'react.titanium.Root',

      children: [],

      add: function add(child) {
        this.children.push(child);
      },

      open: function open(props) {
        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = _getIterator(this.children), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var child = _step2.value;

            child.open(props);
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2['return']) {
              _iterator2['return']();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }
      }
    };
  }
});